'use strict'

var MapHelper = require('app/map-helper');
var Utils = require('app/utils');
var GeoJSON = require('app/geo-json');
var C = require('app/consts');

module.exports = Marionette.ItemView.extend({

	template: _.template($('#event-explorer-view-tpl').html()),

	ui: {
		playEventButton: '#play-event',
		pauseEventButton: '#pause-event',
		repeatCheckbox: '#repeat-event',
		tracker: '#progress-tracker',
		startDate: '#start-date',
		eventSelector: '#event-selector',
		exportButton: '#export-event'
	},

	events: {
		'click @ui.playEventButton': 'play',
		'click @ui.pauseEventButton': 'pause',
		'click @ui.repeatCheckbox': 'toggleRepeat',
		'change @ui.eventSelector': 'loadEvent'
	},

	initialize: function() {
		this.intervalId = 0;
		this.currentFrame = 0;
		this.currentProgress = 0;
		this.framesLength = 100 // in ms
		this.repeat = false;
		this.markerDisplayLen = 120 //in minutes
		this.estimateFramesPerCall = 40;
		this.estimateData = [];
		this.polygon; 
		this.geoJson;
		this.playAnimation;
		this.mapHelper = new MapHelper();
	},

	loadEstimates: function() {
		this.estimateData = [];
		$.ajax(C.get('host') + '/events/' + this.currentEvent.get('id') + '/dates')
		.done(function(dates) {
			for(var i = 0; i < dates.length - 1; i++) {
				var currentStart = moment(dates[i]);
				var currentEnd = moment(dates[i+1]);
				var callsToResolveBeforeStarting = Math.ceil(2 * dates.length / 3);
				$.ajax(C.get('host') + '/estimates?event=' + this.currentEvent.get('id') +
					'&start_date=' + encodeURIComponent(currentStart.format(Utils.defaultDateFmtWithTZ())) + 
					'&end_date=' + encodeURIComponent(currentEnd.format(Utils.defaultDateFmtWithTZ())) + 
					'&forecast_lead=0')
				.done(function(data) {
					this.estimateData = this.estimateData.concat(data);
					callsToResolveBeforeStarting--;
					if(callsToResolveBeforeStarting == 0) {
						this.makePlayable();
					}
				}.bind(this));
			}
		}.bind(this));

	},

	makePlayable: function() {
        this.updatePlayer('playable');
		this.ui.startDate.text(moment(this.startDate).format(Utils.defaultDateFmt()));
	},

	loadEvent: function() {
		var currentEventId = parseInt(this.ui.eventSelector.val());
		this.currentEvent = this.collection.find({id: currentEventId});
		this.ui.exportButton.removeClass('hide');
		this.ui.exportButton.prop('href', this.currentEvent.get('gif'))

		this.resetPlayer();
		if(this.currentEvent) {
			var $twitterAjax = $.ajax(C.get('host') + '/tweets?event=' + this.currentEvent.get('id'))
			.done(function(twitterData) {
		        this.twitterData = twitterData;
		        this.startDate = this.currentEvent.get('start_date');
		        this.initTweets();
			}.bind(this));
			this.loadEstimates();
		}
	},

	initTweets: function() {
		_.each(this.twitterData, function(tweet) {
			this.mapHelper.placeMarker(tweet.location, {hide: true, extras: tweet});
		}, this);
	},

	resetPlayer: function() {
		clearInterval(this.intervalId);
		this.clearMap();
		this.currentFrame = 0;
		this.updatePlayer('notPlayable');
		this.stopAnimatingTracker();
		this.resetTracker();
	},

	pause: function() {
		this.updatePlayer('pause');
		clearInterval(this.intervalId);
		this.stopAnimatingTracker();
	},

	play: function() {
		this.startAnimateTracker();
		this.updatePlayer('play');
		if(this.currentFrame === 0) this.renderFirstFrame();
	    this.renderRemainingFrames();
	},

	renderFirstFrame: function() {
		var google_options = {
	        strokeColor: "#9E78EF", //light purple
	        strokeWeight: 0,
	        strokeOpacity: 0.55,
	        fillColor: "#9E78EF",
	        fillOpacity: 0.55,
        };

        this.geoJson = new GeoJSON(this.estimateData[0].contour, google_options);

    	this.polygon = this.geoJson.geometryToGoogleMaps()[0];
        this.polygon.setMap(Aur.Page.map);
        this.currentFrame++;
    },

	renderRemainingFrames: function() {
		this.intervalId = setInterval(function() {
    		this.renderFrame();
        }.bind(this), this.framesLength);
	},

	renderFrame: function() {
		this.updateMap();
		this.updateDateTime();
		if(this.currentFrame == this.estimateData.length -1) {
			this.handlePlayEnd();
		}
		this.currentFrame++;
    },

    updateMap: function() {
    	this.updateEstimate();
    	this.updateTweets();
    },

    clearMap: function() {
    	this.mapHelper.removeAllMarkers();
    	if(this.polygon) this.polygon.setMap(null);
    },

    updateTweets: function() {
		var currentFrameDate = this.getCurrentFrameDate();
    	var trailingFrameDate = currentFrameDate.clone().subtract('minutes', this.markerDisplayLen);

    	var shouldDisplay = function(tweet) {
    		var tweetDate = moment(tweet.created_at);
			return tweetDate.isBefore(currentFrameDate) && tweetDate.isAfter(trailingFrameDate); 
    	};
    	this.mapHelper.filterMarkers(shouldDisplay);
    },

    updateEstimate: function() {
    	var paths = this.geoJson.calcMultiPolygonPaths(this.estimateData[this.currentFrame].contour.coordinates[0]);
		this.polygon.setPaths(paths);
    },

    updateDateTime: function() {
    	this.ui.startDate.text(this.getCurrentFrameDate().format(Utils.defaultDateFmt()));
    },

    getCurrentFrameDate: function() {
    	return moment(this.startDate).add('minutes', 15 * this.currentFrame);
    },

	handlePlayEnd: function() {
		this.stopAnimatingTracker();
		this.resetTracker();
		this.ui.startDate.text(moment(this.startDate).format(Utils.defaultDateFmt()));
		this.currentFrame = 1;
		if(this.repeat) {
			this.startAnimateTracker();
		}
		else {
			clearInterval(this.intervalId);
			this.updatePlayer('playable');
		}
	},

	startAnimateTracker: function() {
		var ms = this.getEventLengthRemaining();
		this.playAnimation = this.ui.tracker.animate({
		    'margin-left': '90%'
		}, ms), 'linear';
	},

	stopAnimatingTracker: function() {
		if(this.playAnimation) this.playAnimation.stop(true);
	},

	resetTracker: function() {
		this.ui.tracker.css('margin-left', '0%');
	},

	getEventLengthRemaining: function() {
		var frames = _.reduce(this.estimateData, function(memo, dataset){ 
			return memo + dataset.contour.coordinates.length;
		}, 0);
		return (this.framesLength - this.currentFrame) * frames;
	},

	updatePlayer: function(mode) {
		switch(mode) {
			case 'notPlayable':
				this.updateControl(this.ui.playEventButton, false);
				this.updateControl(this.ui.pauseEventButton, false);
				break;
			case 'playable':
				this.updateControl(this.ui.playEventButton, true);
				this.updateControl(this.ui.pauseEventButton, false);
				break;
			case 'play':
				this.updateControl(this.ui.playEventButton, false);
				this.updateControl(this.ui.pauseEventButton, true);
				break;
			case 'pause':
				this.updateControl(this.ui.playEventButton, true);
				this.updateControl(this.ui.pauseEventButton, false);
				break;
		}

	},

	updateControl: function(control, enabledFlag) {
		if(enabledFlag) control.prop('disabled', false);
		else control.prop('disabled', true);
	},

	toggleRepeat: function() {
		this.repeat = this.ui.repeatCheckbox.is(':checked');
	}
});