'use strict';

var GeoJSON = function( geoJson, options ){
	this.geoJson = geoJson;
	this.options = options;
};

_.extend(GeoJSON.prototype, {
	_geometryToGoogleMaps: function( geojsonGeometry, opts, geojsonProperties ){
	
		var googleObj;
		
		switch ( geojsonGeometry.type ){
			case "Point":
				opts.position = new google.maps.LatLng(geojsonGeometry.coordinates[1], geojsonGeometry.coordinates[0]);
				googleObj = new google.maps.Marker(opts);
				if (geojsonProperties) {
					googleObj.set("geojsonProperties", geojsonProperties);
				}
				break;
				
			case "MultiPoint":				
				googleObj = [];
				for (var i = 0; i < geojsonGeometry.coordinates.length; i++){
					opts.position = new google.maps.LatLng(geojsonGeometry.coordinates[i][1], geojsonGeometry.coordinates[i][0]);
					googleObj.push(new google.maps.Marker(opts));
				}
				if (geojsonProperties) {
					for (var k = 0; k < googleObj.length; k++){
						googleObj[k].set("geojsonProperties", geojsonProperties);
					}
				}
				break;
				
			case "LineString":
				var path = [];
				for (var i = 0; i < geojsonGeometry.coordinates.length; i++){
					var coord = geojsonGeometry.coordinates[i];
					var ll = new google.maps.LatLng(coord[1], coord[0]);
					path.push(ll);
				}
				opts.path = path;
				googleObj = new google.maps.Polyline(opts);
				if (geojsonProperties) {
					googleObj.set("geojsonProperties", geojsonProperties);
				}
				break;
				
			case "MultiLineString":
				googleObj = [];
				for (var i = 0; i < geojsonGeometry.coordinates.length; i++){
					var path = [];
					for (var j = 0; j < geojsonGeometry.coordinates[i].length; j++){
						var coord = geojsonGeometry.coordinates[i][j];
						var ll = new google.maps.LatLng(coord[1], coord[0]);
						path.push(ll);
					}
					opts.path = path;
					googleObj.push(new google.maps.Polyline(opts));
				}
				if (geojsonProperties) {
					for (var k = 0; k < googleObj.length; k++){
						googleObj[k].set("geojsonProperties", geojsonProperties);
					}
				}
				break;
				
			case "Polygon":
				var paths = this.calcPolygonPaths(geojsonGeometry);
				opts.paths = paths;
				googleObj = new google.maps.Polygon(opts);
				if (geojsonProperties) {
					googleObj.set("geojsonProperties", geojsonProperties);
				}
				break;
				
			case "MultiPolygon":
				googleObj = [];
				for (var i = 0; i < geojsonGeometry.coordinates.length; i++){
					opts.paths = this.calcMultiPolygonPaths(geojsonGeometry.coordinates[i]);
					googleObj.push(new google.maps.Polygon(opts));
				}
				if (geojsonProperties) {
					for (var k = 0; k < googleObj.length; k++){
						googleObj[k].set("geojsonProperties", geojsonProperties);
					}
				}
				break;
				
			case "GeometryCollection":
				googleObj = [];
				if (!geojsonGeometry.geometries){
					googleObj = this._error('Invalid GeoJSON object: GeometryCollection object missing "geometries" member.');
				}else{
					for (var i = 0; i < geojsonGeometry.geometries.length; i++){
						googleObj.push(this._geometryToGoogleMaps(geojsonGeometry.geometries[i], opts, geojsonProperties || null));
					}
				}
				break;
				
			default:
				googleObj = this._error("Invalid GeoJSON object: Geometry object must be one of \"Point\", \"LineString\", \"Polygon\" or \"MultiPolygon\".");
		}
		
		return googleObj;
		
	},

	calcMultiPolygonPaths: function(coordinates) {
		var paths = [];
		var exteriorDirection;
		var interiorDirection;
		for (var j = 0; j < coordinates.length; j++){
			var path = [];
			for (var k = 0; k < coordinates[j].length; k++){
				var ll = new google.maps.LatLng(coordinates[j][k][1], coordinates[j][k][0]);
				path.push(ll);
			}
			if(!j){
				exteriorDirection = this._ccw(path);
				paths.push(path);
			}else if(j == 1){
				interiorDirection = this._ccw(path);
				if(exteriorDirection == interiorDirection){
					paths.push(path.reverse());
				}else{
					paths.push(path);
				}
			}else{
				if(exteriorDirection == interiorDirection){
					paths.push(path.reverse());
				}else{
					paths.push(path);
				}
			}
		}
		return paths;
	},

	calcPolygonPaths: function(geojsonGeometry) {
		var paths = [];
		var exteriorDirection;
		var interiorDirection;
		for (var i = 0; i < geojsonGeometry.coordinates.length; i++){
			var path = [];
			for (var j = 0; j < geojsonGeometry.coordinates[i].length; j++){
				var ll = new google.maps.LatLng(geojsonGeometry.coordinates[i][j][1], geojsonGeometry.coordinates[i][j][0]);
				path.push(ll);
			}
			if(!i){
				exteriorDirection = this._ccw(path);
				paths.push(path);
			}else if(i == 1){
				interiorDirection = this._ccw(path);
				if(exteriorDirection == interiorDirection){
					paths.push(path.reverse());
				}else{
					paths.push(path);
				}
			}else{
				if(exteriorDirection == interiorDirection){
					paths.push(path.reverse());
				}else{
					paths.push(path);
				}
			}
		}
		return paths;
	},

	_error: function( message ){

		return {
			type: "Error",
			message: message
		};
	
	},

	_ccw: function( path ){
		var isCCW;
		var a = 0;
		for (var i = 0; i < path.length-2; i++){
			a += ((path[i+1].lat() - path[i].lat()) * (path[i+2].lng() - path[i].lng()) - (path[i+2].lat() - path[i].lat()) * (path[i+1].lng() - path[i].lng()));
		}
		if(a > 0){
			isCCW = true;
		}
		else{
			isCCW = false;
		}
		return isCCW;
	},

	geometryToGoogleMaps: function() {
		var obj;
		var opts = this.options || {};
		
		var geojson = this.geoJson;
		switch ( geojson.type ){

			case "FeatureCollection":
				if (!geojson.features){
					obj = this._error('Invalid GeoJSON object: FeatureCollection object missing "features" member.');
				}else{
					obj = [];
					for (var i = 0; i < geojson.features.length; i++){
						obj.push(this._geometryToGoogleMaps(geojson.features[i].geometry, opts, geojson.features[i].properties));
					}
				}
				break;
			
			case "GeometryCollection":
				if (!geojson.geometries){
					obj = this._error('Invalid GeoJSON object: GeometryCollection object missing "geometries" member.');
				}else{
					obj = [];
					for (var i = 0; i < geojson.geometries.length; i++){
						obj.push(this._geometryToGoogleMaps(geojson.geometries[i], opts));
					}
				}
				break;
			
			case "Feature":
				if (!( geojson.properties && geojson.geometry )){
					obj = this._error('Invalid GeoJSON object: Feature object missing "properties" or "geometry" member.');
				}else{
					obj = this._geometryToGoogleMaps(geojson.geometry, opts, geojson.properties);
				}
				break;
			
			case "Point": case "MultiPoint": case "LineString": case "MultiLineString": case "Polygon": case "MultiPolygon":
				obj = geojson.coordinates
					? obj = this._geometryToGoogleMaps(geojson, opts)
					: this._error('Invalid GeoJSON object: Geometry object missing "coordinates" member.');
				break;
			
			default:
				obj = this._error('Invalid GeoJSON object: GeoJSON object must be one of' +
				 '"Point", "LineString", "Polygon", "MultiPolygon", "Feature", "FeatureCollection" or "GeometryCollection".');
		}
		
		return obj;
	}
});
	
module.exports = GeoJSON;