'use strict';

var C = require('app/consts');
var App = require('app/app');
var Map = require('app/map');

var MapHelper = function() {
    this.cluster;
    this.markersData = [];
    this.infoWindow = null;
    this.currentConditions;
    this.cloudCover;

    var map = new Map();
    this.map = map.getMap();
};

_.extend(MapHelper.prototype, Backbone.Events, {

    // Input is either an array of {latitude: ..., longitude...}
    // Or array of mixed objects and arrays
    // ex [ [{latitude: ..., longitude...}, {date: .., somethingElse:... }], {latitude: ..., longitude...} ]
    addMarkers: function(data, config) {
        _.each(data, function(obj) {
            var marker = this._createMarker(obj.location, null, config);
            this._bindInfoBoxListener(marker, obj);
            this.markersData.push(marker);
        }, this);
    },

    placeMarker: function(location, config) {
        var marker = this._createMarker(location, config); //, options);
        var extras = config ? config.extras : null;
        this.markersData.push({
            marker: marker,
            extras: extras
        });;
    },

    onMarker: function(type, fn) {
        _.each(this.markersData, function(markerData) {
            this._addEventListener(type, markerData, fn);
        }, this);
    },

    renderCluster: function(observations, clusterOptions, markerOptions) {
        _.each(observations, function(ob) {
            var marker = this._createMarker(ob.location, null, markerOptions);
            this._bindInfoBoxListener(marker, ob);
            this.markersData.push(marker);
        }, this);
        var mcOptions = clusterOptions || {
            gridSize: C.get('CLUSTER_GRID_SIZE'),
            maxZoom: 15,
            minimumClusterSize: C.get('CLUSTER_SIZE')
        };
        this.cluster = new MarkerClusterer(this.map, this.markersData, mcOptions);
    },

    removeAllMarkers: function() {
        _.each(this.markersData, function(markerData) {
            this.removeMarker(markerData.marker);
        }, this);
        this.markersData = [];
    },

    filterMarkers: function(fn) {
        _.each(this.markersData, function(markersData) {
            if(!markersData.extras) return;
            if(fn(markersData.extras)) {
                if(!markersData.marker.getMap()) {
                    this._showMarker(markersData.marker);
                }
            } else {
                this.removeMarker(markersData.marker);
            }
        }, this);
    },

    updateIcon: function(marker, icon) {
        var mark = _.find(this.markersData, function(m) { return m === marker; });
        if(mark) mark.setIcon(icon);
    },

    removeMarker: function(marker) {
        marker.setMap(null);
    },

    removeCluster: function() { 
        if(this.cluster && this.cluster.clearMarkers) this.cluster.clearMarkers();
        this.markersData = [];
    },

    showCloudCover: function() {
        this.cloudCover = new google.maps.weather.CloudLayer();
        this.cloudCover.setMap(this.map);
    },

    showCurrentConditions: function() {
        this.currentConditions = new google.maps.weather.WeatherLayer({
            temperatureUnits: google.maps.weather.TemperatureUnit.FAHRENHEIT
        });
        this.currentConditions.setMap(this.map);
    },

    showDayNightLayer: function() {
        this.dayNightLayer = new DayNightOverlay({
            map: this.map,
            fillColor: 'rgba(0,0,0,0.3)',
            date: new Date()
        });
    },

    createHeatmap: function(dataArr, options) {
        var latLngs = [];
        _.each(dataArr, function(data) {
            if(data.location && data.location) {
                latLngs.push(new google.maps.LatLng(data.location.latitude, data.location.longitude));
            }
        });

        this.latLngs = latLngs;

        this.heatmap = new google.maps.visualization.HeatmapLayer(options);
        this.heatmap.setData(latLngs);
        this.heatmap.setMap(this.map);
    },

    removeCurrentConditions: function() {
    	this.currentConditions.setMap(null);
    },

    removeCloudCover: function() {
    	this.cloudCover.setMap(null);
    },

    removeDayNightLayer: function() {
        this.dayNightLayer.setMap(null);
    },

    removeHeatmapLayer: function() {
        this.heatmap.setMap(null);
    },

    removeScientistLayer: function() {
        _.each(this.markersData, function(marker) {
            marker.setMap(null);
        });
    },

    _bindInfoBoxListener: function(marker, content) {
        google.maps.event.addListener(marker, 'click', function() {
            this.trigger('markerClicked', {content: content, marker: marker});
        }.bind(this));
    },

    _showMarker: function(marker) {
        marker.setMap(this.map);
    },

    _createMarkers: function(markers, config, options) {
        _.each(markers, function(marker){
            this._createMarker(marker, config, options);
        }, this);
    },

    _createMarker: function(location, config, options) {
        var map = (config && config.hide) ? null : this.map;
        var latLong = new google.maps.LatLng(location.latitude, location.longitude);
        var markerOpts = { 
            position: latLong,
            map: map
        };

        _.each(_.keys(options), function(key) {
            markerOpts[key] = options[key];
        });
        return new google.maps.Marker(markerOpts);
    },

    _addEventListener: function(event_type, markerData, fn) {
        google.maps.event.addListener(markerData.marker, event_type, function() {
            fn(markerData);
        });
    }

});

module.exports = MapHelper;